@@ -56,8 +56,8 @@
 #include "RpcMethodFactory.h"
 #include "RpcRequest.h"
 #include "RpcResponse.h"
+#include "rpc_helper.h"
 #ifdef ENABLE_XML_RPC
-# include "XmlRpcRequestProcessor.h"
 # include "XmlRpcRequestParserStateMachine.h"
 #endif // ENABLE_XML_RPC
 
@@ -86,21 +86,6 @@ HttpServerBodyCommand::~HttpServerBodyCommand()
   e_->deleteSocketForReadCheck(socket_, this);
 }
 
-namespace {
-rpc::RpcResponse
-createJsonRpcErrorResponse
-(int code,
- const std::string& msg,
- const SharedHandle<ValueBase>& id)
-{
-  SharedHandle<Dict> params = Dict::g();
-  params->put("code", Integer::g(code));
-  params->put("message", msg);
-  rpc::RpcResponse res(code, params, id);
-  return res;
-}
-} // namespace
-
 namespace {
 std::string getJsonRpcContentType(bool script)
 {
@@ -113,22 +98,22 @@ void HttpServerBodyCommand::sendJsonRpcResponse
  const std::string& callback)
 {
   bool gzip = httpServer_->supportsGZip();
-  std::string responseData = res.toJson(callback, gzip);
+  std::string responseData = rpc::toJson(res, callback, gzip);
   if(res.code == 0) {
     httpServer_->feedResponse(responseData,
                               getJsonRpcContentType(!callback.empty()));
   } else {
     httpServer_->disableKeepAlive();
-    std::string httpCode;
+    int httpCode;
     switch(res.code) {
     case -32600:
-      httpCode = "400 Bad Request";
+      httpCode = 400;
       break;
     case -32601:
-      httpCode = "404 Not Found";
+      httpCode = 404;
       break;
     default:
-      httpCode = "500 Internal Server Error";
+      httpCode = 500;
     };
     httpServer_->feedResponse(httpCode, A2STR::NIL,
                               responseData,
@@ -156,42 +141,6 @@ void HttpServerBodyCommand::addHttpServerResponseCommand()
   e_->setNoWait(true);
 }
 
-rpc::RpcResponse
-HttpServerBodyCommand::processJsonRpcRequest(const Dict* jsondict)
-{
-
-  SharedHandle<ValueBase> id = jsondict->get("id");
-  if(!id) {
-    return createJsonRpcErrorResponse(-32600, "Invalid Request.", Null::g());
-  }
-  const String* methodName = asString(jsondict->get("method"));
-  if(!methodName) {
-    return createJsonRpcErrorResponse(-32600, "Invalid Request.", id);
-  }
-  SharedHandle<List> params;
-  const SharedHandle<ValueBase>& tempParams = jsondict->get("params");
-  if(asList(tempParams)) {
-    params = static_pointer_cast<List>(tempParams);
-  } else if(!tempParams) {
-    params = List::g();
-  } else {
-    // TODO No support for Named params
-    return createJsonRpcErrorResponse(-32602, "Invalid params.", id);
-  }
-  rpc::RpcRequest req(methodName->s(), params, id);
-  req.jsonRpc = true;
-  SharedHandle<rpc::RpcMethod> method;
-  try {
-    method = rpc::RpcMethodFactory::create(req.methodName);
-  } catch(RecoverableException& e) {
-    A2_LOG_INFO_EX(EX_EXCEPTION_CAUGHT, e);
-    return createJsonRpcErrorResponse(-32601, "Method not found.", id);
-  }
-  A2_LOG_INFO(fmt("Executing RPC method %s", req.methodName.c_str()));
-  rpc::RpcResponse res = method->execute(req, e_);
-  return res;
-}
-
 bool HttpServerBodyCommand::execute()
 {
   if(e_->getRequestGroupMan()->downloadFinished() || e_->isHaltRequested()) {
@@ -201,7 +150,7 @@ bool HttpServerBodyCommand::execute()
     if(socket_->isReadable(0) ||
        !httpServer_->getSocketRecvBuffer()->bufferEmpty() ||
        httpServer_->getContentLength() == 0) {
-      timeoutTimer_ = global::wallclock;
+      timeoutTimer_ = global::wallclock();
 
       if(httpServer_->receiveBody()) {
         std::string reqPath = httpServer_->getRequestPath();
@@ -213,17 +162,31 @@ bool HttpServerBodyCommand::execute()
         // Do something for requestpath and body
         if(reqPath == "/rpc") {
 #ifdef ENABLE_XML_RPC
-          rpc::RpcRequest req =
-            rpc::XmlRpcRequestProcessor().parseMemory(httpServer_->getBody());
+          std::string body = httpServer_->getBody();
+          rpc::RpcRequest req;
+          try {
+            req = rpc::xmlParseMemory(body.c_str(), body.size());
+          } catch(RecoverableException& e) {
+            A2_LOG_INFO_EX
+              (fmt("CUID#%lld - Failed to parse XML-RPC request",
+                   getCuid()),
+               e);
+            httpServer_->feedResponse(400);
+            addHttpServerResponseCommand();
+            return true;
+          }
           SharedHandle<rpc::RpcMethod> method =
             rpc::RpcMethodFactory::create(req.methodName);
           A2_LOG_INFO(fmt("Executing RPC method %s", req.methodName.c_str()));
           rpc::RpcResponse res = method->execute(req, e_);
           bool gzip = httpServer_->supportsGZip();
-          std::string responseData = res.toXml(gzip);
+          std::string responseData = rpc::toXml(res, gzip);
           httpServer_->feedResponse(responseData, "text/xml");
           addHttpServerResponseCommand();
-#endif // ENABLE_XML_RPC
+#else // !ENABLE_XML_RPC
+          httpServer_->feedResponse(404);
+          addHttpServerResponseCommand();
+#endif // !ENABLE_XML_RPC
           return true;
         } else if(reqPath == "/jsonrpc") {
           std::string callback;
@@ -242,37 +205,40 @@ bool HttpServerBodyCommand::execute()
                    getCuid()),
                e);
             rpc::RpcResponse res
-              (createJsonRpcErrorResponse(-32700, "Parse error.", Null::g()));
+              (rpc::createJsonRpcErrorResponse(-32700, "Parse error.",
+                                               Null::g()));
             sendJsonRpcResponse(res, callback);
             return true;
           }
-          const Dict* jsondict = asDict(json);
+          const Dict* jsondict = downcast<Dict>(json);
           if(jsondict) {
-            rpc::RpcResponse res = processJsonRpcRequest(jsondict);
+            rpc::RpcResponse res = rpc::processJsonRpcRequest(jsondict, e_);
             sendJsonRpcResponse(res, callback);
           } else {
-            const List* jsonlist = asList(json);
+            const List* jsonlist = downcast<List>(json);
             if(jsonlist) {
               // This is batch call
               std::vector<rpc::RpcResponse> results;
               for(List::ValueType::const_iterator i = jsonlist->begin(),
                     eoi = jsonlist->end(); i != eoi; ++i) {
-                const Dict* jsondict = asDict(*i);
+                const Dict* jsondict = downcast<Dict>(*i);
                 if(jsondict) {
-                  rpc::RpcResponse r = processJsonRpcRequest(jsondict);
+                  rpc::RpcResponse r = rpc::processJsonRpcRequest(jsondict, e_);
                   results.push_back(r);
                 }
               }
               sendJsonRpcBatchResponse(results, callback);
             } else {
               rpc::RpcResponse res
-                (createJsonRpcErrorResponse
+                (rpc::createJsonRpcErrorResponse
                  (-32600, "Invalid Request.", Null::g()));
               sendJsonRpcResponse(res, callback);
             }
           }
           return true;
         } else {
+          httpServer_->feedResponse(404);
+          addHttpServerResponseCommand();
           return true;
         }
       } else {
@@ -280,7 +246,7 @@ bool HttpServerBodyCommand::execute()
         return false;
       } 
     } else {
-      if(timeoutTimer_.difference(global::wallclock) >= 30) {
+      if(timeoutTimer_.difference(global::wallclock()) >= 30) {
         A2_LOG_INFO("HTTP request body timeout.");
         return true;
       } else {
