@@ -43,18 +43,7 @@
 namespace aria2 {
 
 class MetalinkParserStateMachine;
-
-struct XmlAttr {
-  std::string localname;
-  std::string prefix;
-  std::string nsUri;
-  std::string value;
-
-  XmlAttr();
-  XmlAttr(const XmlAttr& attr);
-  ~XmlAttr();
-  XmlAttr& operator=(const XmlAttr&);
-};
+struct XmlAttr;
 
 class MetalinkParserState
 {
@@ -63,16 +52,16 @@ public:
 
   virtual void beginElement
   (MetalinkParserStateMachine* stm,
-   const std::string& localname,
-   const std::string& prefix,
-   const std::string& nsUri,
+   const char* localname,
+   const char* prefix,
+   const char* nsUri,
    const std::vector<XmlAttr>& attrs) {}
   
   virtual void endElement
   (MetalinkParserStateMachine* stm,
-   const std::string& localname,
-   const std::string& prefix,
-   const std::string& nsUri,
+   const char* localname,
+   const char* prefix,
+   const char* nsUri,
    const std::string& characters) {}
 
   virtual bool needsCharactersBuffering() const
