@@ -8,6 +8,8 @@
 #include "Segment.h"
 #include "Option.h"
 #include "PieceSelector.h"
+#include "FileEntry.h"
+#include "PeerStat.h"
 
 namespace aria2 {
 
@@ -59,19 +61,20 @@ void SegmentManTest::testNullBitfield()
   SharedHandle<UnknownLengthPieceStorage> ps
     (new UnknownLengthPieceStorage(dctx, &op));
   SegmentMan segmentMan(&op, dctx, ps);
+  size_t minSplitSize = dctx->getPieceLength();
 
-  SharedHandle<Segment> segment = segmentMan.getSegment(1);
-  CPPUNIT_ASSERT(!segment.isNull());
+  SharedHandle<Segment> segment = segmentMan.getSegment(1, minSplitSize);
+  CPPUNIT_ASSERT(segment);
   CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getIndex());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getLength());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getSegmentLength());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getWrittenLength());
+  CPPUNIT_ASSERT_EQUAL(0, segment->getLength());
+  CPPUNIT_ASSERT_EQUAL(0, segment->getSegmentLength());
+  CPPUNIT_ASSERT_EQUAL(0, segment->getWrittenLength());
 
-  SharedHandle<Segment> segment2 = segmentMan.getSegment(2);
-  CPPUNIT_ASSERT(segment2.isNull());
+  SharedHandle<Segment> segment2 = segmentMan.getSegment(2, minSplitSize);
+  CPPUNIT_ASSERT(!segment2);
 
   segmentMan.cancelSegment(1);
-  CPPUNIT_ASSERT(!segmentMan.getSegment(2).isNull());
+  CPPUNIT_ASSERT(segmentMan.getSegment(2, minSplitSize));
 }
 
 void SegmentManTest::testCompleteSegment()
@@ -85,10 +88,10 @@ void SegmentManTest::testCompleteSegment()
 
   SegmentMan segmentMan(&op, dctx, ps);
 
-  CPPUNIT_ASSERT(!segmentMan.getSegment(1, 0).isNull());
-  SharedHandle<Segment> seg = segmentMan.getSegment(1, 1);
-  CPPUNIT_ASSERT(!seg.isNull());
-  CPPUNIT_ASSERT(!segmentMan.getSegment(1, 2).isNull());
+  CPPUNIT_ASSERT(segmentMan.getSegmentWithIndex(1, 0));
+  SharedHandle<Segment> seg = segmentMan.getSegmentWithIndex(1, 1);
+  CPPUNIT_ASSERT(seg);
+  CPPUNIT_ASSERT(segmentMan.getSegmentWithIndex(1, 2));
 
   seg->updateWrittenLength(pieceLength);
   segmentMan.completeSegment(1, seg);
@@ -113,31 +116,31 @@ void SegmentManTest::testGetSegment_sameFileEntry()
   dctx->setFileEntries(&fileEntries[0], &fileEntries[3]);
   SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, &op));
   SegmentMan segman(&op, dctx, ps);
-
+  size_t minSplitSize =dctx->getPieceLength();
   std::vector<SharedHandle<Segment> > segments;
-  segman.getSegment(segments, 1, fileEntries[1], 4);
+  segman.getSegment(segments, 1, minSplitSize, fileEntries[1], 4);
   // See 3 segments are returned, not 4 because the part of file1 is
   // not filled in segment#1
   CPPUNIT_ASSERT_EQUAL((size_t)3, segments.size());
   
-  SharedHandle<Segment> segmentNo1 = segman.getSegment(2, 1);
+  SharedHandle<Segment> segmentNo1 = segman.getSegmentWithIndex(2, 1);
   // Fill the part of file1 in segment#1
   segmentNo1->updateWrittenLength(1);
   segman.cancelSegment(2);
 
   segman.cancelSegment(1);
   segments.clear();
-  segman.getSegment(segments, 1, fileEntries[1], 4);
+  segman.getSegment(segments, 1, minSplitSize, fileEntries[1], 4);
   CPPUNIT_ASSERT_EQUAL((size_t)4, segments.size());
 
   segman.cancelSegment(1);
-  SharedHandle<Segment> segmentNo4 = segman.getSegment(1, 4);
+  SharedHandle<Segment> segmentNo4 = segman.getSegmentWithIndex(1, 4);
   // Fill the part of file2 in segment#4
   segmentNo4->updateWrittenLength(1);
   segman.cancelSegment(1);
 
   segments.clear();
-  segman.getSegment(segments, 1, fileEntries[1], 4);
+  segman.getSegment(segments, 1, minSplitSize, fileEntries[1], 4);
   // segment#4 is not returned because the part of file2 is filled.
   CPPUNIT_ASSERT_EQUAL((size_t)3, segments.size());
 }
@@ -162,13 +165,13 @@ void SegmentManTest::testRegisterPeerStat()
 
 void SegmentManTest::testCancelAllSegments()
 {
-  segmentMan_->getSegment(1, 0);
-  segmentMan_->getSegment(2, 1);
-  CPPUNIT_ASSERT(segmentMan_->getSegment(3, 0).isNull());
-  CPPUNIT_ASSERT(segmentMan_->getSegment(4, 1).isNull());
+  segmentMan_->getSegmentWithIndex(1, 0);
+  segmentMan_->getSegmentWithIndex(2, 1);
+  CPPUNIT_ASSERT(!segmentMan_->getSegmentWithIndex(3, 0));
+  CPPUNIT_ASSERT(!segmentMan_->getSegmentWithIndex(4, 1));
   segmentMan_->cancelAllSegments();
-  CPPUNIT_ASSERT(!segmentMan_->getSegment(3, 0).isNull());
-  CPPUNIT_ASSERT(!segmentMan_->getSegment(4, 1).isNull());
+  CPPUNIT_ASSERT(segmentMan_->getSegmentWithIndex(3, 0));
+  CPPUNIT_ASSERT(segmentMan_->getSegmentWithIndex(4, 1));
 }
 
 void SegmentManTest::testGetPeerStat()
@@ -180,20 +183,20 @@ void SegmentManTest::testGetPeerStat()
 
 void SegmentManTest::testGetCleanSegmentIfOwnerIsIdle()
 {
-  SharedHandle<Segment> seg1 = segmentMan_->getSegment(1, 0);
-  SharedHandle<Segment> seg2 = segmentMan_->getSegment(2, 1);
+  SharedHandle<Segment> seg1 = segmentMan_->getSegmentWithIndex(1, 0);
+  SharedHandle<Segment> seg2 = segmentMan_->getSegmentWithIndex(2, 1);
   seg2->updateWrittenLength(100);
-  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(3, 0).isNull());
+  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(3, 0));
   SharedHandle<PeerStat> peerStat3(new PeerStat(3));
   segmentMan_->registerPeerStat(peerStat3);
-  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(4, 0).isNull());
+  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(4, 0));
   SharedHandle<PeerStat> peerStat4(new PeerStat(4));
   peerStat4->downloadStart();
   segmentMan_->registerPeerStat(peerStat4);
   // Owner PeerStat is not IDLE
-  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 0).isNull());
+  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 0));
   // Segment::updateWrittenLength != 0
-  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 1).isNull());
+  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 1));
 }
 
 } // namespace aria2
